import loader from './loader';
import ramlClient from './ramlClient';

export default class Sandbox {
  constructor() {
    if (typeof document !== 'undefined') {
      this.frame = document.createElement('iframe');
      this.frame.style.display = 'none';
      this.executeWindow = {};
      this.frame.addEventListener('load', () => {
        this.frame.contentWindow.executeWindow = this.executeWindow;
      });

      document.body.appendChild(this.frame);
    }
  }

  execute(code, blockNumber) {
    let promise = Promise.resolve();
    const exec = {
      blockNumber
    };

    try {
      exec.result = this.frame.contentWindow.eval(`with(executeWindow){\n${code}\n}`);
    } catch (error) {
      exec.error = error;
    }

    if (exec.result && exec.result.then && typeof exec.result.then === 'function') {
      promise = exec.result
        .then((value) => {
          exec.result = value;
        })
        .catch((error) => {
          exec.error = error;
        });
    }

    return promise
      .then(() => {
        if (blockNumber !== undefined) {
          this.executeWindow[`$${blockNumber}`] = exec.error || exec.result;
        }

        return exec;
      });
  }

  register(editor, clientConfig) {
    if (this.isRegistered) {
      return;
    }
    this.isRegistered = true;

    this.executeWindow.API = ramlClient(editor, this.executeWindow, clientConfig);
    this.executeWindow.load = (src, done) => loader(this, src, done);
  }

  dispose() {
    if (this.frame.parentNode) {
      this.frame.parentNode.removeChild(this.frame);
    }

    this.isDisposed = true;
  }
}
