import PropTypes from 'prop-types';
import React from 'react';
import Notebook from './component';
import Sandbox from './sandbox';

export default class StandaloneApp extends React.Component {
  static propTypes = {
    content: PropTypes.string,
    blockNumber: PropTypes.number,
    loaderPath: PropTypes.string.isRequired,
    sandbox: PropTypes.object,
    onChange: PropTypes.func,
    beforeExecute: PropTypes.func,
    ramlClientConfig: PropTypes.object
  };

  static defaultProps = {
    content: '',
    blockNumber: 0,
    sandbox: new Sandbox(),
    onChange: () => {},
    beforeExecute: () => Promise.resolve()
  };

  constructor(props) {
    super(props);

    this.state = {
      notebookState: {
        content: this.props.content
      }
    };
  }

  handleChange = (notebookState, afterChange = () => {}) => {
    this.setState({
      notebookState: {
        ...notebookState,
        isExecuted: false
      }
    }, () => {
      this.props.onChange(this.state.notebookState);
      afterChange(this.state.notebookState);
    });
  }

  handleExecute = () => {
    const {
      notebookState
    } = this.state;
    const {
      blockNumber,
      sandbox,
      beforeExecute
    } = this.props;

    this.handleChange({ ...notebookState, isLoading: true });

    return beforeExecute(notebookState)
      .then(() => (
        new Promise((resolve, reject) => {
          process.nextTick(() => {
            sandbox.execute(notebookState.content, blockNumber)
              .then((exec) => {
                this.handleChange({ ...notebookState, exec, isLoading: false }, (newNotebookState) => {
                  this.setState({ notebookState: { ...newNotebookState, isExecuted: true } });
                });
              })
              .then(resolve)
              .catch(reject);
          });
        })
      ));
  }

  render() {
    const {
      blockNumber,
      loaderPath,
      sandbox,
      ramlClientConfig
    } = this.props;
    const {
      notebookState
    } = this.state;

    return (
      <Notebook
        blockNumber={blockNumber}
        loaderPath={loaderPath}
        notebookState={notebookState}
        onChange={this.handleChange}
        onExecute={this.handleExecute}
        sandbox={sandbox}
        ramlClientConfig={ramlClientConfig}
      />
    );
  }
}
